{% load helpers %}

<ul class="nav nav-tabs" style="margin-bottom: 20px">
    <li role="presentation"{% if active_tab == 'add' %} class="active"{% endif %}>
        <a href="{% url 'plugins:nautobot_cable_utils:cabletemplate_add' %}{% querystring request %}">New Cable Template</a>
    </li>
    <li role="presentation"{% if active_tab == 'bulk_add' %} class="active"{% endif %}>
        <a href="{% url 'plugins:nautobot_cable_utils:cabletemplate_bulk_add' %}{% querystring request %}">Bulk Create</a>
    </li>
</ul>
