# Generated by Django 3.2.13 on 2022-08-08 09:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('nautobot_cable_utils', '0002_auto_20220808_0747'),
    ]

    operations = [
        migrations.RenameField(
            model_name='cabletemplate',
            old_name='tenant',
            new_name='owner',
        ),
    ]
