{% if cable_planned %}
<a href="{% url 'plugins:nautobot_cable_utils:device_commission' device.pk %}" class="btn btn-success">
    <span class="glyphicon glyphicon-th-list" aria-hidden="true"></span> Device Commission
</a>
{% endif %}
